@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2013 Andrew M. Bishop
+ This file Copyright 2008-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -52,7 +52,7 @@
 int option_quiet=0;
 
 /*+ The options to select the format of the output. +*/
-int option_html=0,option_gpx_track=0,option_gpx_route=0,option_text=0,option_text_all=0,option_none=0;
+int option_html=0,option_gpx_track=0,option_gpx_route=0,option_text=0,option_text_all=0,option_none=0,option_stdout=0;
 
 /*+ The option to calculate the quickest route insted of the shortest. +*/
 int option_quickest=0;
@@ -61,6 +61,7 @@ int option_quickest=0;
 /* Local functions */
 
 static void print_usage(int detail,const char *argerr,const char *err);
+static Results *CalculateRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,Profile *profile,index_t start_node,index_t prev_segment,index_t finish_node);
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -69,24 +70,26 @@ static void print_usage(int detail,const char *argerr,const char *err);
 
 int main(int argc,char** argv)
 {
- Nodes    *OSMNodes;
- Segments *OSMSegments;
- Ways     *OSMWays;
- Relations*OSMRelations;
- Results  *results[NWAYPOINTS+1]={NULL};
- int       point_used[NWAYPOINTS+1]={0};
- double    point_lon[NWAYPOINTS+1],point_lat[NWAYPOINTS+1];
- double    heading=-999;
- int       help_profile=0,help_profile_xml=0,help_profile_json=0,help_profile_pl=0;
- char     *dirname=NULL,*prefix=NULL;
- char     *profiles=NULL,*profilename=NULL;
- char     *translations=NULL,*language=NULL;
- int       exactnodes=0;
- Transport transport=Transport_None;
- Profile  *profile=NULL;
- index_t   start_node=NO_NODE,finish_node=NO_NODE;
- index_t   join_segment=NO_SEGMENT;
- int       arg,point;
+ Nodes     *OSMNodes;
+ Segments  *OSMSegments;
+ Ways      *OSMWays;
+ Relations *OSMRelations;
+ Results   *results[NWAYPOINTS+1]={NULL};
+ int        point_used[NWAYPOINTS+1]={0};
+ double     point_lon[NWAYPOINTS+1],point_lat[NWAYPOINTS+1];
+ double     heading=-999;
+ int        help_profile=0,help_profile_xml=0,help_profile_json=0,help_profile_pl=0;
+ char      *dirname=NULL,*prefix=NULL;
+ char      *profiles=NULL,*profilename=NULL;
+ char      *translations=NULL,*language=NULL;
+ int        exactnodes=0,reverse=0,loop=0;
+ Transport  transport=Transport_None;
+ Profile   *profile=NULL;
+ index_t    start_node,finish_node=NO_NODE,first_node=NO_NODE;
+ index_t    join_segment=NO_SEGMENT;
+ int        arg,nresults=0;
+ waypoint_t start_waypoint,finish_waypoint=NO_WAYPOINT;
+ waypoint_t first_waypoint=NWAYPOINTS,last_waypoint=1,inc_dec_waypoint,waypoint;
 
  /* Parse the command line arguments */
 
@@ -117,6 +120,10 @@ int main(int argc,char** argv)
        translations=&argv[arg][15];
     else if(!strcmp(argv[arg],"--exact-nodes-only"))
        exactnodes=1;
+    else if(!strcmp(argv[arg],"--reverse"))
+       reverse=1;
+    else if(!strcmp(argv[arg],"--loop"))
+       loop=1;
     else if(!strcmp(argv[arg],"--quiet"))
        option_quiet=1;
     else if(!strcmp(argv[arg],"--loggable"))
@@ -133,6 +140,8 @@ int main(int argc,char** argv)
        option_text_all=1;
     else if(!strcmp(argv[arg],"--output-none"))
        option_none=1;
+    else if(!strcmp(argv[arg],"--output-stdout"))
+      { option_stdout=1; option_quiet=1; }
     else if(!strncmp(argv[arg],"--profile=",10))
        profilename=&argv[arg][10];
     else if(!strncmp(argv[arg],"--language=",11))
@@ -150,6 +159,12 @@ int main(int argc,char** argv)
     argv[arg]=NULL;
    }
 
+ if(option_stdout && (option_html+option_gpx_track+option_gpx_route+option_text+option_text_all)!=1)
+   {
+    fprintf(stderr,"Error: The '--output-stdout' option requires exactly one other output option (but not '--output-none').\n");
+    exit(EXIT_FAILURE);
+   }
+
  /* Load in the profiles */
 
  if(transport==Transport_None)
@@ -213,52 +228,47 @@ int main(int argc,char** argv)
        option_quickest=0;
     else if(!strcmp(argv[arg],"--quickest"))
        option_quickest=1;
-    else if(isdigit(argv[arg][0]) ||
-       ((argv[arg][0]=='-' || argv[arg][0]=='+') && isdigit(argv[arg][1])))
+    else if(!strncmp(argv[arg],"--lon",5) && isdigit(argv[arg][5]))
       {
-       for(point=1;point<=NWAYPOINTS;point++)
-          if(point_used[point]!=3)
-            {
-             if(point_used[point]==0)
-               {
-                point_lon[point]=degrees_to_radians(atof(argv[arg]));
-                point_used[point]=1;
-               }
-             else /* if(point_used[point]==1) */
-               {
-                point_lat[point]=degrees_to_radians(atof(argv[arg]));
-                point_used[point]=3;
-               }
-             break;
-            }
-      }
-     else if(!strncmp(argv[arg],"--lon",5) && isdigit(argv[arg][5]))
-       {
-        char *p=&argv[arg][6];
-        while(isdigit(*p)) p++;
-        if(*p++!='=')
-           print_usage(0,argv[arg],NULL);
+       int point;
+       char *p=&argv[arg][6];
+
+       while(isdigit(*p)) p++;
+       if(*p++!='=')
+          print_usage(0,argv[arg],NULL);
  
-        point=atoi(&argv[arg][5]);
-        if(point>NWAYPOINTS || point_used[point]&1)
-           print_usage(0,argv[arg],NULL);
+       point=atoi(&argv[arg][5]);
+       if(point>NWAYPOINTS || point_used[point]&1)
+          print_usage(0,argv[arg],NULL);
  
        point_lon[point]=degrees_to_radians(atof(p));
        point_used[point]+=1;
+
+       if(point<first_waypoint)
+          first_waypoint=point;
+       if(point>last_waypoint)
+          last_waypoint=point;
       }
-     else if(!strncmp(argv[arg],"--lat",5) && isdigit(argv[arg][5]))
-       {
-        char *p=&argv[arg][6];
-        while(isdigit(*p)) p++;
-        if(*p++!='=')
-           print_usage(0,argv[arg],NULL);
+    else if(!strncmp(argv[arg],"--lat",5) && isdigit(argv[arg][5]))
+      {
+       int point;
+       char *p=&argv[arg][6];
+
+       while(isdigit(*p)) p++;
+       if(*p++!='=')
+          print_usage(0,argv[arg],NULL);
  
-        point=atoi(&argv[arg][5]);
-        if(point>NWAYPOINTS || point_used[point]&2)
-           print_usage(0,argv[arg],NULL);
+       point=atoi(&argv[arg][5]);
+       if(point>NWAYPOINTS || point_used[point]&2)
+          print_usage(0,argv[arg],NULL);
  
        point_lat[point]=degrees_to_radians(atof(p));
        point_used[point]+=2;
+
+       if(point<first_waypoint)
+          first_waypoint=point;
+       if(point>last_waypoint)
+          last_waypoint=point;
       }
     else if(!strncmp(argv[arg],"--heading=",10))
       {
@@ -352,8 +362,8 @@ int main(int argc,char** argv)
        print_usage(0,argv[arg],NULL);
    }
 
- for(point=1;point<=NWAYPOINTS;point++)
-    if(point_used[point]==1 || point_used[point]==2)
+ for(waypoint=1;waypoint<=NWAYPOINTS;waypoint++)
+    if(point_used[waypoint]==1 || point_used[waypoint]==2)
        print_usage(0,NULL,"All waypoints must have latitude and longitude.");
 
  /* Print one of the profiles if requested */
@@ -434,45 +444,68 @@ int main(int argc,char** argv)
     exit(EXIT_FAILURE);
    }
 
- /* Loop through all pairs of points */
+ /* Check for reverse direction */
 
- for(point=1;point<=NWAYPOINTS;point++)
+ if(reverse)
+   {
+    waypoint_t temp;
+
+    temp=first_waypoint;
+    first_waypoint=last_waypoint;
+    last_waypoint=temp;
+
+    last_waypoint--;
+
+    inc_dec_waypoint=-1;
+   }
+ else
+   {
+    last_waypoint++;
+
+    inc_dec_waypoint=1;
+   }
+
+ /* Loop through all pairs of waypoints */
+
+ for(waypoint=first_waypoint;waypoint!=last_waypoint;waypoint+=inc_dec_waypoint)
    {
-    Results *begin,*end;
     distance_t distmax=km_to_distance(MAXSEARCH);
     distance_t distmin;
     index_t segment=NO_SEGMENT;
     index_t node1,node2;
 
-    if(point_used[point]!=3)
+    if(point_used[waypoint]!=3)
        continue;
 
     /* Find the closest point */
 
     start_node=finish_node;
+    start_waypoint=finish_waypoint;
 
     if(exactnodes)
       {
-       finish_node=FindClosestNode(OSMNodes,OSMSegments,OSMWays,point_lat[point],point_lon[point],distmax,profile,&distmin);
+       finish_node=FindClosestNode(OSMNodes,OSMSegments,OSMWays,point_lat[waypoint],point_lon[waypoint],distmax,profile,&distmin);
       }
     else
       {
        distance_t dist1,dist2;
 
-       segment=FindClosestSegment(OSMNodes,OSMSegments,OSMWays,point_lat[point],point_lon[point],distmax,profile,&distmin,&node1,&node2,&dist1,&dist2);
+       segment=FindClosestSegment(OSMNodes,OSMSegments,OSMWays,point_lat[waypoint],point_lon[waypoint],distmax,profile,&distmin,&node1,&node2,&dist1,&dist2);
 
        if(segment!=NO_SEGMENT)
-          finish_node=CreateFakes(OSMNodes,OSMSegments,point,LookupSegment(OSMSegments,segment,1),node1,node2,dist1,dist2);
+          finish_node=CreateFakes(OSMNodes,OSMSegments,waypoint,LookupSegment(OSMSegments,segment,1),node1,node2,dist1,dist2);
        else
           finish_node=NO_NODE;
       }
 
     if(finish_node==NO_NODE)
       {
-       fprintf(stderr,"Error: Cannot find node close to specified point %d.\n",point);
+       fprintf(stderr,"Error: Cannot find node close to specified point %d.\n",waypoint);
        exit(EXIT_FAILURE);
       }
 
+    finish_waypoint=waypoint;
+
     if(!option_quiet)
       {
        double lat,lon;
@@ -483,127 +516,49 @@ int main(int argc,char** argv)
           GetLatLong(OSMNodes,finish_node,NULL,&lat,&lon);
 
        if(IsFakeNode(finish_node))
-          printf("Point %d is segment %"Pindex_t" (node %"Pindex_t" -> %"Pindex_t"): %3.6f %4.6f = %2.3f km\n",point,segment,node1,node2,
+          printf("Waypoint %d is segment %"Pindex_t" (node %"Pindex_t" -> %"Pindex_t"): %3.6f %4.6f = %2.3f km\n",waypoint,segment,node1,node2,
                  radians_to_degrees(lon),radians_to_degrees(lat),distance_to_km(distmin));
        else
-          printf("Point %d is node %"Pindex_t": %3.6f %4.6f = %2.3f km\n",point,finish_node,
+          printf("Waypoint %d is node %"Pindex_t": %3.6f %4.6f = %2.3f km\n",waypoint,finish_node,
                  radians_to_degrees(lon),radians_to_degrees(lat),distance_to_km(distmin));
       }
 
+    /* Check the nodes */
+
     if(start_node==NO_NODE)
        continue;
 
     if(start_node==finish_node)
        continue;
 
+    if(first_node==NO_NODE)
+       first_node=start_node;
+
     if(heading!=-999 && join_segment==NO_SEGMENT)
        join_segment=FindClosestSegmentHeading(OSMNodes,OSMSegments,OSMWays,start_node,heading,profile);
 
-    /* Calculate the beginning of the route */
+    /* Calculate the route */
 
-    begin=FindStartRoutes(OSMNodes,OSMSegments,OSMWays,OSMRelations,profile,start_node,join_segment,finish_node);
+    results[nresults]=CalculateRoute(OSMNodes,OSMSegments,OSMWays,OSMRelations,profile,start_node,join_segment,finish_node);
 
-    if(begin)
-      {
-       /* Check if the end of the route was reached */
+    results[nresults]->start_waypoint=start_waypoint;
+    results[nresults]->finish_waypoint=finish_waypoint;
 
-       if(begin->finish_node!=NO_NODE)
-          results[point]=ExtendStartRoutes(OSMNodes,OSMSegments,OSMWays,OSMRelations,profile,begin,finish_node);
-      }
-    else
-      {
-       if(join_segment!=NO_SEGMENT)
-         {
-          /* Try again but allow a U-turn at the start waypoint -
-             this solves the problem of facing a dead-end that contains no super-nodes. */
+    join_segment=results[nresults]->last_segment;
 
-          join_segment=NO_SEGMENT;
+    nresults++;
+   }
 
-          begin=FindStartRoutes(OSMNodes,OSMSegments,OSMWays,OSMRelations,profile,start_node,join_segment,finish_node);
-         }
+ /* Finish the loop */
 
-       if(begin)
-         {
-          /* Check if the end of the route was reached */
+ if(loop && finish_node!=NO_NODE)
+   {
+    results[nresults]=CalculateRoute(OSMNodes,OSMSegments,OSMWays,OSMRelations,profile,finish_node,join_segment,first_node);
 
-          if(begin->finish_node!=NO_NODE)
-             results[point]=ExtendStartRoutes(OSMNodes,OSMSegments,OSMWays,OSMRelations,profile,begin,finish_node);
-         }
-       else
-         {
-          fprintf(stderr,"Error: Cannot find initial section of route compatible with profile.\n");
-          exit(EXIT_FAILURE);
-         }
-      }
+    results[nresults]->start_waypoint=last_waypoint;
+    results[nresults]->finish_waypoint=first_waypoint;
 
-    /* Calculate the rest of the route */
-
-    if(!results[point])
-      {
-       Results *middle;
-
-       /* Calculate the end of the route */
-
-       end=FindFinishRoutes(OSMNodes,OSMSegments,OSMWays,OSMRelations,profile,finish_node);
-
-       if(!end)
-         {
-          fprintf(stderr,"Error: Cannot find final section of route compatible with profile.\n");
-          exit(EXIT_FAILURE);
-         }
-
-       /* Calculate the middle of the route */
-
-       middle=FindMiddleRoute(OSMNodes,OSMSegments,OSMWays,OSMRelations,profile,begin,end);
-
-       if(!middle && join_segment!=NO_SEGMENT)
-         {
-          /* Try again but allow a U-turn at the start waypoint -
-             this solves the problem of facing a dead-end that contains some super-nodes. */
-
-          FreeResultsList(begin);
-
-          begin=FindStartRoutes(OSMNodes,OSMSegments,OSMWays,OSMRelations,profile,start_node,NO_SEGMENT,finish_node);
-
-          if(begin)
-             middle=FindMiddleRoute(OSMNodes,OSMSegments,OSMWays,OSMRelations,profile,begin,end);
-         }
-
-       FreeResultsList(end);
-
-       if(!middle)
-         {
-          fprintf(stderr,"Error: Cannot find super-route compatible with profile.\n");
-          exit(EXIT_FAILURE);
-         }
-
-       results[point]=CombineRoutes(OSMNodes,OSMSegments,OSMWays,OSMRelations,profile,begin,middle);
-
-       if(!results[point])
-         {
-          fprintf(stderr,"Error: Cannot create combined route following super-route.\n");
-          exit(EXIT_FAILURE);
-         }
-
-       FreeResultsList(begin);
-
-       FreeResultsList(middle);
-      }
-
-#if DEBUG
-    Result *r=FindResult(results[point],results[point]->start_node,results[point]->prev_segment);
-
-    printf("The final route is:\n");
-
-    while(r)
-      {
-       printf("  node=%"Pindex_t" segment=%"Pindex_t" score=%f\n",r->node,r->segment,r->score);
-
-       r=r->next;
-      }
-#endif
-
-    join_segment=results[point]->last_segment;
+    nresults++;
    }
 
  if(!option_quiet)
@@ -615,15 +570,14 @@ int main(int argc,char** argv)
  /* Print out the combined route */
 
  if(!option_none)
-    PrintRoute(results,NWAYPOINTS,OSMNodes,OSMSegments,OSMWays,profile);
+    PrintRoute(results,nresults,OSMNodes,OSMSegments,OSMWays,profile);
 
  /* Destroy the remaining results lists and data structures */
 
 #if 0
 
- for(point=1;point<=NWAYPOINTS;point++)
-    if(results[point])
-       FreeResultsList(results[point]);
+ for(waypoint=0;waypoint<=nresults;waypoint++)
+    FreeResultsList(results[waypoint]);
 
  DestroyNodeList(OSMNodes);
  DestroySegmentList(OSMSegments);
@@ -636,6 +590,140 @@ int main(int argc,char** argv)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Find a complete route from a specified node to another node.
+
+  Results *CalculateRoute Returns a set of results.
+
+  Nodes *nodes The set of nodes to use.
+
+  Segments *segments The set of segments to use.
+
+  Ways *ways The set of ways to use.
+
+  Relations *relations The set of relations to use.
+
+  Profile *profile The profile containing the transport type, speeds and allowed highways.
+
+  index_t start_node The start node.
+
+  index_t prev_segment The previous segment before the start node.
+
+  index_t finish_node The finish node.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static Results *CalculateRoute(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,Profile *profile,index_t start_node,index_t prev_segment,index_t finish_node)
+{
+ Results *begin,*end,*complete=NULL;
+
+ /* Calculate the beginning of the route */
+
+ begin=FindStartRoutes(nodes,segments,ways,relations,profile,start_node,prev_segment,finish_node);
+
+ if(begin)
+   {
+    /* Check if the end of the route was reached */
+
+    if(begin->finish_node!=NO_NODE)
+       complete=ExtendStartRoutes(nodes,segments,ways,relations,profile,begin,finish_node);
+   }
+ else
+   {
+    if(prev_segment!=NO_SEGMENT)
+      {
+       /* Try again but allow a U-turn at the start waypoint -
+          this solves the problem of facing a dead-end that contains no super-nodes. */
+
+       prev_segment=NO_SEGMENT;
+
+       begin=FindStartRoutes(nodes,segments,ways,relations,profile,start_node,prev_segment,finish_node);
+      }
+
+    if(begin)
+      {
+       /* Check if the end of the route was reached */
+
+       if(begin->finish_node!=NO_NODE)
+          complete=ExtendStartRoutes(nodes,segments,ways,relations,profile,begin,finish_node);
+      }
+    else
+      {
+       fprintf(stderr,"Error: Cannot find initial section of route compatible with profile.\n");
+       exit(EXIT_FAILURE);
+      }
+   }
+
+ /* Calculate the rest of the route */
+
+ if(!complete)
+   {
+    Results *middle;
+
+    /* Calculate the end of the route */
+
+    end=FindFinishRoutes(nodes,segments,ways,relations,profile,finish_node);
+
+    if(!end)
+      {
+       fprintf(stderr,"Error: Cannot find final section of route compatible with profile.\n");
+       exit(EXIT_FAILURE);
+      }
+
+    /* Calculate the middle of the route */
+
+    middle=FindMiddleRoute(nodes,segments,ways,relations,profile,begin,end);
+
+    if(!middle && prev_segment!=NO_SEGMENT)
+      {
+       /* Try again but allow a U-turn at the start waypoint -
+          this solves the problem of facing a dead-end that contains some super-nodes. */
+
+       FreeResultsList(begin);
+
+       begin=FindStartRoutes(nodes,segments,ways,relations,profile,start_node,NO_SEGMENT,finish_node);
+
+       if(begin)
+          middle=FindMiddleRoute(nodes,segments,ways,relations,profile,begin,end);
+      }
+
+    FreeResultsList(end);
+
+    if(!middle)
+      {
+       fprintf(stderr,"Error: Cannot find super-route compatible with profile.\n");
+       exit(EXIT_FAILURE);
+      }
+
+    complete=CombineRoutes(nodes,segments,ways,relations,profile,begin,middle);
+
+    if(!complete)
+      {
+       fprintf(stderr,"Error: Cannot create combined route following super-route.\n");
+       exit(EXIT_FAILURE);
+      }
+
+    FreeResultsList(begin);
+
+    FreeResultsList(middle);
+   }
+
+#if DEBUG
+ Result *r=FindResult(complete,complete->start_node,complete->prev_segment);
+
+ printf("The final route is:\n");
+
+ while(r)
+   {
+    printf("  node=%"Pindex_t" segment=%"Pindex_t" score=%f\n",r->node,r->segment,r->score);
+
+    r=r->next;
+   }
+#endif
+
+ return(complete);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Print out the usage information.
 
@@ -659,13 +747,14 @@ static void print_usage(int detail,const char *argerr,const char *err)
          "              [--output-html]\n"
          "              [--output-gpx-track] [--output-gpx-route]\n"
          "              [--output-text] [--output-text-all]\n"
-         "              [--output-none]\n"
+         "              [--output-none] [--output-stdout]\n"
          "              [--profile=<name>]\n"
          "              [--transport=<transport>]\n"
          "              [--shortest | --quickest]\n"
          "              --lon1=<longitude> --lat1=<latitude>\n"
          "              --lon2=<longitude> --lon2=<latitude>\n"
          "              [ ... --lon99=<longitude> --lon99=<latitude>]\n"
+         "              [--reverse] [--loop]\n"
          "              [--highway-<highway>=<preference> ...]\n"
          "              [--speed-<highway>=<speed> ...]\n"
          "              [--property-<property>=<preference> ...]\n"
@@ -716,6 +805,8 @@ static void print_usage(int detail,const char *argerr,const char *err)
             "--output-text-all       Write a plain test file with all route points.\n"
             "--output-none           Don't write any output files or read any translations.\n"
             "                        (If no output option is given then all are written.)\n"
+            "--output-stdout         Write to stdout instead of a file (requires exactly\n"
+            "                        one output format option, implies '--quiet').\n"
             "\n"
             "--profile=<name>        Select the loaded profile with this name.\n"
             "--transport=<transport> Select the transport to use (selects the profile\n"
@@ -727,6 +818,9 @@ static void print_usage(int detail,const char *argerr,const char *err)
             "--lon<n>=<longitude>    Specify the longitude of the n'th waypoint.\n"
             "--lat<n>=<latitude>     Specify the latitude of the n'th waypoint.\n"
             "\n"
+            "--reverse               Find a route between the waypoints in reverse order.\n"
+            "--loop                  Find a route that returns to the first waypoint.\n"
+            "\n"
             "--heading=<bearing>     Initial compass bearing at lowest numbered waypoint.\n"
             "\n"
             "                                   Routing preference options\n"
