@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2013 Andrew M. Bishop
+ This file Copyright 2008-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -39,6 +39,9 @@
 #define NWAYPOINTS 99
 
 
+/*+ An undefined waypoint index. +*/
+#define NO_WAYPOINT    ((waypoint_t)~0)
+
 /*+ An undefined node index. +*/
 #define NO_NODE        ((index_t)~0)
 
@@ -128,6 +131,10 @@
 /* Simple Types */
 
 
+/*+ A waypoint index. +*/
+typedef uint16_t waypoint_t;
+
+
 /*+ A node, segment, way or relation index. +*/
 typedef uint32_t index_t;
 
@@ -179,10 +186,10 @@ typedef uint16_t ll_off_t;
 /*+ Node flags. +*/
 typedef uint16_t nodeflags_t;
 
-/*+ A distance, measured in metres. +*/
+/*+ A distance, measured in metres (will not overflow for any earth-based distance). +*/
 typedef uint32_t distance_t;
 
-/*+ A duration, measured in 1/10th seconds. +*/
+/*+ A duration, measured in 1/10th seconds (will not overflow for 13 years). +*/
 typedef uint32_t duration_t;
 
 /*+ A routing optimisation score. +*/
@@ -331,8 +338,9 @@ typedef enum _Property
   Property_Tunnel       = 4,
   Property_FootRoute    = 5,
   Property_BicycleRoute = 6,
+  Property_CycleBothWays= 7,
 
-  Property_Count        = 7       /* One more than the number of property types. */
+  Property_Count        = 8       /* One more than the number of property types. */
  }
  Property;
 
@@ -347,14 +355,15 @@ typedef enum _Properties
  {
   Properties_None         = 0,
 
-  Properties_Paved        = PROPERTIES(Property_Paved       ),
-  Properties_Multilane    = PROPERTIES(Property_Multilane   ),
-  Properties_Bridge       = PROPERTIES(Property_Bridge      ),
-  Properties_Tunnel       = PROPERTIES(Property_Tunnel      ),
-  Properties_FootRoute    = PROPERTIES(Property_FootRoute   ),
-  Properties_BicycleRoute = PROPERTIES(Property_BicycleRoute),
+  Properties_Paved        = PROPERTIES(Property_Paved        ),
+  Properties_Multilane    = PROPERTIES(Property_Multilane    ),
+  Properties_Bridge       = PROPERTIES(Property_Bridge       ),
+  Properties_Tunnel       = PROPERTIES(Property_Tunnel       ),
+  Properties_FootRoute    = PROPERTIES(Property_FootRoute    ),
+  Properties_BicycleRoute = PROPERTIES(Property_BicycleRoute ),
+  Properties_CycleBothWays= PROPERTIES(Property_CycleBothWays),
 
-  Properties_ALL          = PROPERTIES(Property_Count       )-1
+  Properties_ALL          = PROPERTIES(Property_Count        )-1
  }
  Properties;
 
@@ -375,32 +384,37 @@ typedef uint8_t width_t;
 typedef uint8_t length_t;
 
 
-/*+ Conversion of km/hr to speed_t. +*/
-#define kph_to_speed(xxx)      (speed_t)(xxx)
+/*+ Conversion of km/hr to speed_t - simple inline function with error checking. +*/
+inline static speed_t kph_to_speed(double xxx);
+inline static speed_t kph_to_speed(double xxx) { if(xxx>255) return(255); if(xxx<0) return(0); return((speed_t)xxx); }
 
 /*+ Conversion of speed_t to km/hr. +*/
 #define speed_to_kph(xxx)      (int)(xxx)
 
-/*+ Conversion of tonnes to weight_t. +*/
-#define tonnes_to_weight(xxx)  (weight_t)((xxx)*5)
+/*+ Conversion of tonnes to weight_t - simple inline function with error checking. +*/
+inline static weight_t tonnes_to_weight(double xxx);
+inline static weight_t tonnes_to_weight(double xxx) { if(xxx>51) return(255); if(xxx<0) return(0); return((weight_t)(xxx*5)); }
 
 /*+ Conversion of weight_t to tonnes. +*/
 #define weight_to_tonnes(xxx)  ((double)(xxx)/5.0)
 
-/*+ Conversion of metres to height_t. +*/
-#define metres_to_height(xxx)  (height_t)((xxx)*10)
+/*+ Conversion of metres to height_t - simple inline function with error checking. +*/
+inline static height_t metres_to_height(double xxx);
+inline static height_t metres_to_height(double xxx) { if(xxx>25.5) return(255); if(xxx<0) return(0); return((height_t)(xxx*10)); }
 
 /*+ Conversion of height_t to metres. +*/
 #define height_to_metres(xxx)  ((double)(xxx)/10.0)
 
-/*+ Conversion of metres to width_t. +*/
-#define metres_to_width(xxx)   (width_t)((xxx)*10)
+/*+ Conversion of metres to width_t - simple inline function with error checking. +*/
+inline static width_t metres_to_width(double xxx);
+inline static width_t metres_to_width(double xxx) { if(xxx>25.5) return(255); if(xxx<0) return(0); return((width_t)(xxx*10)); }
 
 /*+ Conversion of width_t to metres. +*/
 #define width_to_metres(xxx)   ((double)(xxx)/10.0)
 
-/*+ Conversion of metres to length_t. +*/
-#define metres_to_length(xxx)  (length_t)((xxx)*10)
+/*+ Conversion of metres to length_t - simple inline function with error checking. +*/
+inline static length_t metres_to_length(double xxx);
+inline static length_t metres_to_length(double xxx) { if(xxx>25.5) return(255); if(xxx<0) return(0); return((length_t)(xxx*10)); }
 
 /*+ Conversion of length_t to metres. +*/
 #define length_to_metres(xxx)  ((double)(xxx)/10.0)
